//東京の、2015年における1日ごとの平均気温平均の風速を表した棒グラフを
//円状にレイアウトした。平均気温は色、風速は棒の長さで表している

String[] date; //日付
float[] temp;    //気温
float[] wind;  //風速
float highTemp;  //最高気温
float lowTemp;  //最低気温
PFont myFont;    //フォント用変数
float x1, y1, x2, y2;  //月ごとの分割用の線を描画するための変数
int month;
XML xml;

void setup() {
  size(500, 500);
  colorMode(HSB, 360, 100, 100);  //色彩の表示方法をHSBに変更
  loadData();  //xmlファイルからデータ読み込み、オブジェクト化

  //一番低い気温と高い気温をtemp配列内で検索し、lowTemp、highTempに入れる
  lowTemp = min(temp);
  highTemp = max(temp);

  myFont = createFont("Arial", 12);    //フォント作成
  textFont(myFont, 14);  //テキストのサイズを設定
  textAlign(CENTER);
}

void draw() {
  background(0, 0, 100);  //背景は白

  stroke(0, 0, 80);
  fill(0, 0, 100);
  ellipse(width/2, height/2, 480, 480);  //外側の円

  //月ごとの分割線を描画。1ヶ月ごとにグレーの線を描いていく///////////////
  //文字を表示するときにrotateを使うと、文字も回転してしまうので、
  //ここではsin, cosを使っている。

  for (int i = 0; i < 12; i ++) {

    x1 = 50*cos(radians(i*30));  //30度ごとのグレーの線の内側の点を求める
    y1 = 50*sin(radians(i*30));

    x2 = 240*cos(radians(i*30));  //30度ごとのグレーの線の外側の点を求める
    y2 = 240*sin(radians(i*30));

    //線を描画
    line(width/2 + x1, height/2 + y1, width/2 + x2, height/2 + y2);

    //月をテキストで書く。一番上から時計回りで始めるため、90度引く。
    //さらに、15度移動すれば、グレーの線の間に表示することができる
    x1 = 200*cos(radians(i*30 + 15 - 90));
    y1 = 200*sin(radians(i*30 + 15 - 90));

    month = i + 1;  //月は、i + 1で1月から始まる
    fill(0, 0, 50);
    text(month + "月", width/2 + x1, height/2 + y1);  //月を描画
  } 

  //気温と風速を描画////////////////////////////////////////

  //365日分の1日は、角度にすると360度分の何度になるかを計算 = 0.986.....
  float angle = 360.0/365.0;

  //1日ごとに描画
  for (int i = 0; i < temp.length; i ++) {

    pushMatrix();
    translate(width/2, height/2);  //画面中央に座標系を移動
    rotate(radians(i*angle));  //1日分ずつ座標系を回転

    stroke(createColor(temp[i], lowTemp, highTemp), 100, 100);
    line(0, -50, 0, -50 - wind[i]*28);  //線を描画
    popMatrix();
  }
}

void loadData() {

  xml = loadXML("weather_day.xml");  //ファイルの読み込み

  //weatherタグ内の要素を取得
  XML[] children = xml.getChildren("weather");

  //要素数を取得して配列を生成
  date = new String[children.length];
  temp = new float[children.length];
  wind = new float[children.length];

  //配列に値を入れる
  for (int i = 0; i < temp.length; i++) {

    //月
    XML dateXml = children[i].getChild("date");
    date[i] = dateXml.getContent();

    //気温
    XML tempXml = children[i].getChild("temp");
    temp[i] = tempXml.getFloatContent();

    //風速
    XML windXml = children[i].getChild("wind");
    wind[i] = windXml.getFloatContent();
  }
}

//気温から色を決定
float createColor(float _temp, float _lowTemp, float _highTemp) {

  //最低気温から最高気温の範囲を、色相の200°から0°に変換
  //気温を色に変換
  float barColor = map(_temp, _lowTemp, _highTemp, 200, 0);

  return barColor;
}