float highestTemp = -1000;  //最高気温 //<>//
float lowestTemp = 1000;  //最低気温
float averageTemp = 0;  //平均気温
PFont din12;  //フォントオブジェクト

Weather[] weathers;  //天気オブジェクト
String[] months = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};

void setup() {
  size(300, 550);
  loadData();  //xmlファイルからデータ読み込み、オブジェクト化

  float sum = 0;  //一旦全部の気温を合計する
  for (int i = 0; i < weathers.length; i++) {
    float temp = weathers[i].temp;
    sum += temp;
    if (highestTemp < temp) {  //現状最高気温として登録している気温よりも高い気温に遭遇したら
      highestTemp = temp;  //その気温を最高気温として登録
    }
    if (lowestTemp > temp) {  //現状最低気温として登録している気温よりも低い気温に遭遇したら
      lowestTemp = temp;  //その気温を最低気温として登録
    }
  }
  averageTemp = sum/float(weathers.length);  //全気温合計を365日で割って平均気温を求める

  din12 = loadFont("DINNextLTPro-Light-12.vlw");    //フォント作成
  textFont(din12, 12);  //テキストのサイズを設定
}

boolean drawFlag = true;  //このフレームで描画が必要かどうか

void draw() {

  if (!drawFlag) {  //描画が必要なければ
    return;  //このdraw()関数を終了
  }
  drawFlag = false;  //描画を行うので一旦フラグをOFFにする

  background(255);
  int month = 0;  //現在処理中の月。1月を1とするがとりあえず0から始める。

  for (int i=0; i<weathers.length; i++) {  //365回繰り返す
    Weather weather = weathers[i];  //その日の天気オブジェクトを取り出し

    float temp = weather.temp;  //気温を取り出し
    colorMode(HSB);
    noStroke();
    fill(150 - temp*5, 200, 200);  //Hを150(青)から気温が高いだけ引いていく。

    int sx = 130;  //全体的に描画位置をずらすための変数。0にすると左上から始まる。
    int sy = 30;
    int x = sx + i%6 * 12;  //実際にその日の気温をどこに描画するかの変数。
    int y = sy + floor(i/6) * 8;

    rect(x, y, 10, 6);

    if (weather.month != month) {  //もし一ループの処理と今回の処理が別の月なら
      month = weather.month;  //処理中の月を登録
      fill(0);
      text(months[month-1], sx-40, y + 10);  //JAN,FEBなど、その月の文字列を描画


      fill(0);
      triangle(x, y, x+6, y, x, y+6);  //各月1日目にはわかりやすいように三角形の目印を描画
    }
  }
}

void loadData() {
  //ファイルをロードして配列を作る
  JSONArray weatherData = loadJSONArray("weather_day.json");

  //要素数を取得して配列を生成
  weathers = new Weather[weatherData.size()];  //weatherData.size(365個)分のWeatherオブジェクトの配列

  for (int i = 0; i < weatherData.size(); i++) {  //クロールする

    //一つ一つのJSON配列の中を検索
    JSONObject day = weatherData.getJSONObject(i); 

    weathers[i] = new Weather();  //オブジェクトを作成
    weathers[i].date = day.getString("date");  //それぞれのプロパティを埋めていく。
    String[] dateSplit = split(day.getString("date"), "/");  // "4/3"というような文字列を"4"と"3"に分割するため、"/"を境に文字を分割する。
    weathers[i].month = int(dateSplit[0]);  //分割した結果の一つ目
    weathers[i].day = int(dateSplit[1]);  //二つ目
    weathers[i].temp = day.getFloat("temp");
    weathers[i].wind = day.getFloat("wind");
  }
}

class Weather {  //1日の天気オブジェクト。プロパティのみ。

  String date;
  int month;
  int day;
  float temp;
  float wind;
}