float highestTemp = -1000;  //最高気温 //<>//
float lowestTemp = 1000;  //最低気温
float averageTemp = 0;  //平均気温
PFont din12;  //フォントオブジェクト

Weather[] weathers;  //天気オブジェクト
String[] months = {"JAN", "FEB", "MAR", "APR", "MAY", "JUN", "JUL", "AUG", "SEP", "OCT", "NOV", "DEC"};

void setup() {
  size(1000, 300);
  loadData();  //xmlファイルからデータ読み込み、オブジェクト化

  float sum = 0;  //一旦全部の気温を合計する
  for (int i = 0; i < weathers.length; i++) {
    float temp = weathers[i].temp;
    sum += temp;
    if (highestTemp < temp) {  //現状最高気温として登録している気温よりも高い気温に遭遇したら
      highestTemp = temp;  //その気温を最高気温として登録
    }
    if (lowestTemp > temp) {  //現状最低気温として登録している気温よりも低い気温に遭遇したら
      lowestTemp = temp;  //その気温を最低気温として登録
    }
  }
  averageTemp = sum/float(weathers.length);  //全気温合計を365日で割って平均気温を求める

  din12 = loadFont("DINNextLTPro-Light-12.vlw");    //フォント作成
}



boolean drawFlag = true;  //このフレームで描画が必要かどうか

void draw() {

  if (!drawFlag) {  //描画が必要なければ
    return;  //このdraw()関数を終了
  }
  drawFlag = false;  //描画を行うので一旦フラグをOFFにする


  background(255);

  int tempRange = floor(highestTemp) - floor(lowestTemp) + 1;  //最低気温から最高気温までの範囲
  int temps[][] = new int[12][tempRange];  //それぞれの月の、それぞれの気温の合計日数を記録する配列
  int daysInMonth[] = new int[12];  //それぞれの月が何日まであるかを記録する配列


  for (int i=0; i<weathers.length; i++) {  //1年分毎日クロール
    Weather today = weathers[i];  //今日
    Weather tomorrow = weathers[(i+1)%weathers.length];  //翌日
    temps[today.month - 1][floor(today.temp) - floor(lowestTemp)] += 1;  //今日が存在するの月の、気温の配列をカウントアップ

    if (today.month != tomorrow.month) {  //もし明日が今日と違う月なら
      daysInMonth[today.month-1] = today.day;  //今日が今月最後の日なので、今月が何日まであったかを記録
    }
  }


  float monthWidth = floor(width/12.0)+1;  //1月当たりの幅
  float margin = monthWidth/2.0;  //カンバス左端からのマージン
  float heightRatio = 5.0;  //グラフの縦幅を調整する変数

  for (int i=0; i<11; i++) {  //11回クロール

    int countThisMonth = 0;  //今月の、日数を随時カウントする変数
    int countNextMonth = 0;  //来月分、同上

    for (int j=0; j<tempRange; j++) {  //気温範囲分クロール

      int thisMonth = i;  //今月
      int nextMonth = i+1;  //来月

      float marginThisMonth = height - (height - daysInMonth[thisMonth]*heightRatio)/2.0;  //カンバス下端からのマージン(今月)
      float marginNextMonth = height - (height - daysInMonth[nextMonth]*heightRatio)/2.0;  //カンバス下端からのマージン(来月)

      float xThisMonth = i*monthWidth;  //今月のx座標
      float xNextMonth = (i+1)*monthWidth;  //来月のx座標
      float yThisMonthThisTemp = marginThisMonth - countThisMonth*heightRatio;  //今月の、現在の気温のy座標
      float yNextMonthThisTemp = marginNextMonth - countNextMonth*heightRatio;  //来月の、現在の気温のy座標

      countThisMonth += temps[thisMonth][j];  //今月に存在する、現在の気温だった日数をカウントアップ 
      countNextMonth += temps[nextMonth][j];  //来月に存在する、現在の気温だった日数をカウントアップ
      float yThisMonthNextTemp = floor(marginThisMonth - countThisMonth*heightRatio);  //今月の、次の気温のy座標
      float yNextMonthNextTemp = floor(marginNextMonth - countNextMonth*heightRatio);  //来月の、次の気温のy座標

      if (temps[thisMonth][j] > 0 || temps[nextMonth][j] >0) {  //今月か来月にその気温だった日が1日もなければ描画する必要がない

        float temp = j + floor(lowestTemp);  //気温を取り出し
        colorMode(HSB);
        noStroke();
        fill(150 - temp*5, 200, 200);  //気温に応じた塗りつぶし色

        beginShape();  //4角形を描画
        vertex(margin + xThisMonth, yThisMonthThisTemp);
        vertex(margin + xNextMonth, yNextMonthThisTemp);
        vertex(margin + xNextMonth, yNextMonthNextTemp);
        vertex(margin + xThisMonth, yThisMonthNextTemp);
        endShape();

        colorMode(RGB);
        stroke(255, 255, 255, 50);  //薄い白色
        noFill();
        line(margin + xThisMonth, yThisMonthThisTemp, margin + xNextMonth, yNextMonthThisTemp);  //間をつなぐ線を描画
      }
    }

    fill(0);
    textFont(din12, 12);  //テキストのサイズを設定
    textAlign(CENTER, CENTER);
    text(months[i], margin + monthWidth*i, height/2 + 16*heightRatio + 20 );  //JAN,FEBなど、その月の文字列を描画
  }
  int i=11;
  text(months[i], margin + monthWidth*i, height/2 + 16*heightRatio + 20 );  //12月だけはループ内で処理できないので追加で描画
}

void loadData() {
  //ファイルをロードして配列を作る
  JSONArray weatherData = loadJSONArray("weather_day.json");

  //要素数を取得して配列を生成
  weathers = new Weather[weatherData.size()];  //weatherData.size(365個)分のWeatherオブジェクトの配列

  for (int i = 0; i < weatherData.size(); i++) {  //クロールする

    //一つ一つのJSON配列の中を検索
    JSONObject day = weatherData.getJSONObject(i); 

    weathers[i] = new Weather();  //オブジェクトを作成
    weathers[i].date = day.getString("date");  //それぞれのプロパティを埋めていく。
    String[] dateSplit = split(day.getString("date"), "/");  // "4/3"というような文字列を"4"と"3"に分割するため、"/"を境に文字を分割する。
    weathers[i].month = int(dateSplit[0]);  //分割した結果の一つ目
    weathers[i].day = int(dateSplit[1]);  //二つ目
    weathers[i].temp = day.getFloat("temp");
    weathers[i].wind = day.getFloat("wind");
  }
}

class Weather {  //1日の天気オブジェクト。プロパティのみ。

  String date;
  int month;
  int day;
  float temp;
  float wind;
}