String[] date;//日付 //<>//
float[] temp;//気温
float highTemp;//最高気温
float lowTemp;//最低気温

int montnNum = 12;
Month []monthData = new Month[12];//月ごとのデータを格納するオブジェクト
MyVertex []myVertex = new MyVertex[montnNum];//月ごとのデータを描画するオブジェクト

void setup() {
  size(400, 400);
  loadData();//xmlファイルからデータ読み込み、オブジェクト化
  colorMode(HSB, 360, 100, 100);//HSBモードにする

  lowTemp = min(temp);
  highTemp = max(temp);

  //Monthオブジェクトを生成
  for (int i = 0; i < 12; i++) {
    monthData[i] = new Month();
  }

  //Monthオブジェクトに月ごとのデータを格納していく
  for (int i = 0; i<date.length; i++) {
    String []splitStr = split(date[i], '/');
    int arrNum = Integer.parseInt(splitStr[0])-1;
    monthData[arrNum].setNum(date[i], temp[i]);
  }

  //myVertexオブジェクトを作成し、月ごとのデータを渡す
  for (int i = 0; i < montnNum; i++) {
    myVertex[i] = new MyVertex(monthData[i]);
  }
}

void draw() {

  background(0);
  pushMatrix();//座標保存
  translate(width/2, height/2);//描画の１を中心に

  //月ごとにデータを描画
  for (int i = 0; i < montnNum; i++) {
    myVertex[i].draw();
  }

  popMatrix();//座標解放
}

class MyVertex {
  Month month = new Month();//月ごとのデータ
  ArrayList <PVector>pos = new ArrayList <PVector>();//頂点の位置
  int max;//日数
  float mag = 4.8;
  int randomNum = (int)random(1, 6);//重なり合う線を描画する回数
  ArrayList <Float>randomPos = new ArrayList <Float>();

  MyVertex(Month _month) {
    month = _month;
    max = month.date.size();

    //頂点の位置を格納していく
    float angle = 0;
    for (int i = 0; i < month.date.size(); i++) {

      float temp = month.temp.get(i);
      float x = cos(radians(angle))*temp*1.3;
      float y = sin(radians(angle))*temp*1.3;
      pos.add(new PVector(x, y));
      float oneAngle = 360/ month.date.size();//日数で角度を決める
      angle += oneAngle;//角度を増やしていく
    }

    //重なりあう線の位置を生成
    for (int i = 0; i< randomNum; i++) {
      randomPos.add(random(0.98, 1.02));
    }
  }

  void draw() {
    noStroke();

    //日ごとに円を描画
    for (int i = 0; i < max; i++) {
      fill(createColor(month.temp.get(i), lowTemp, highTemp), 38, 99, 240);
      ellipse(pos.get(i).x*mag, pos.get(i).y*mag, 5, 5);
    }

    curveTightness(0.3);//曲線の曲率を指定
    beginShape(); 

    noFill();

    stroke(239, 38, 99, 100);

    //制御点のための最後の頂点から描画する
    curveVertex(pos.get(max-1).x*mag, pos.get(max-1).y*mag);

    //順番に描画していく
    for (int i = 0; i < pos.size(); i++) {

      curveVertex(pos.get(i).x*mag, pos.get(i).y*mag);
    }

    //閉曲線のため、初めの3点と同じ位置を繰り返す
    curveVertex(pos.get(0).x*mag, pos.get(0).y*mag);
    curveVertex(pos.get(1).x*mag, pos.get(1).y*mag);
    endShape();

    curveTightness(1.0);//曲線の曲率を指定

    //上記と同じく頂点を描画していき、ランダムで少しずらした線を描画する    
    beginShape();
    for (int k = 0; k < randomNum; k++) {
      noFill();

      stroke(239, 38, 99, 70);
      beginShape();
      curveVertex(pos.get(max-1).x*mag*randomPos.get(k), pos.get(max-1).y*mag*randomPos.get(k));
      for (int i = 0; i < pos.size(); i++) {

        curveVertex(pos.get(i).x*mag*randomPos.get(k), pos.get(i).y*mag*randomPos.get(k));
      }
      curveVertex(pos.get(0).x*mag*randomPos.get(k), pos.get(0).y*mag*randomPos.get(k));
      curveVertex(pos.get(1).x*mag*randomPos.get(k), pos.get(1).y*mag*randomPos.get(k));
      endShape();
    }
  }
}

//月ごとのデータを格納する
class Month {
  ArrayList<String> date = new ArrayList<String>();//日付
  ArrayList<Float> temp = new ArrayList<Float>();//気温

  void setNum(String _date, float _temp) {
    date.add(_date);
    temp.add(_temp);
  }
}

void loadData() {
  //ファイルをロードして配列を作る
  JSONArray weatherData = loadJSONArray("weather_day.json");

  //要素数を取得して配列を生成
  date = new String[weatherData.size()];
  temp = new float[weatherData.size()];

  for (int i = 0; i < weatherData.size(); i++) {

    //一つ一つのJSON配列の中を検索
    JSONObject weather = weatherData.getJSONObject(i); 

    //date, temp配列に一つずつ入れる
    date[i] = weather.getString("date");
    temp[i] = weather.getFloat("temp");
  }
}

//気温から色を決定
float createColor(float _temp, float _lowTemp, float _highTemp) {

  //最低気温から最高気温の範囲を、色相の200°から0°に変換
  //気温を色に変換
  float barColor = map(_temp, _lowTemp, _highTemp, 200, 0);

  return barColor;
}