String[] date;//日付 //<>//
float[] temp; //気温
float highTemp;//最高気温
float lowTemp;//最低気温
PFont font;

int monthNum = 12;
Month []monthData = new Month[monthNum];//月ごとのデータを格納するオブジェクト
MyVertex []myVertex = new MyVertex[monthNum];//月ごとのデータを描画するオブジェクト


int barWidth = 420;//スライドバーの横幅
int barHeight = 20;//スライドバーの高さ

int partsWidth = barWidth/monthNum;//パーツの横幅
int barPosX = 90;//スライドバーのX座標
int barPosY;//スライドバーのy座標

int firstPosX = barPosX;//パーツの初期位置
int secondPosX = barPosX+partsWidth*11;//パーツの初期位置

//パーツとマウスの衝突フラグ
boolean isTarget1 = false;
boolean isTarget2 = false;

//パーツの現在位置
int targetPos1 = 0;
int targetPos2 = 11;

boolean isOverlap = false;//パーツが重なり合った時のフラグ


void setup() {
  size(600, 550);
  loadData();//xmlファイルからデータ読み込み、オブジェクト化
  colorMode(HSB, 360, 100, 100);//HSBモードにする

  lowTemp = min(temp);
  highTemp = max(temp);

  //Monthオブジェクトを生成
  for (int i = 0; i < 12; i++) {
    monthData[i] = new Month();
  }

  //Monthオブジェクトに月ごとのデータを格納していく
  for (int i = 0; i<date.length; i++) {
    String []splitStr = split(date[i], '/');
    int arrNum = Integer.parseInt(splitStr[0])-1;
    monthData[arrNum].setNum(date[i], temp[i]);
  }

  //myVertexオブジェクトを作成し、月ごとのデータを渡す
  for (int i = 0; i < monthNum; i++) {
    myVertex[i] = new MyVertex(monthData[i]);
  }

  //スライドバーの高さを設定
  barPosY = height-80;

  //フォントの生成
  font = createFont("Arial", 11);
  textFont(font, 11);
  textAlign(CENTER);
}



void draw() {


  background(0);

  //スライダーのパーツの最小位置設定  
  if (firstPosX < barPosX) {
    firstPosX = barPosX;
  }
  if (secondPosX < barPosX) {
    secondPosX = barPosX;
  }

  //スライダーのパーツの最大位置設定
  if (firstPosX > barWidth + barPosX- partsWidth) {
    firstPosX = barPosX + barWidth - partsWidth;
  }
  if (secondPosX > barWidth + barPosX- partsWidth) {
    secondPosX = barPosX + barWidth - partsWidth;
  }

  noStroke();
  fill(200);

  //スライダーの月ごとのテキストを描画
  for (int i = barPosX; i < partsWidth*monthNum+barPosX; i+=partsWidth) {
    text(String.valueOf((i-barPosX)/partsWidth+1)+"月", i+16, barPosY+barHeight+15);
  }

  //左右のパーツの間を描画する四角
  fill(360, 0, 10);
  rect(firstPosX, barPosY+3, secondPosX-firstPosX+partsWidth, barHeight-6);

  //スライダのパーツ
  fill(239, 38, 99, 110);
  rect(firstPosX, barPosY, partsWidth, barHeight);

  //スライダのパーツ
  fill(239, 38, 99, 110);
  rect(secondPosX, barPosY, partsWidth, barHeight);

  //データの描画
  if (firstPosX == secondPosX) {
    //パーツが重なりあった場合は１つ目の値を使用
    myVertex[targetPos1].draw();
    myVertex[targetPos1].collision();
    isOverlap = true;
  } else {

    //月ごとにデータを描画
    for (int i = min(targetPos1, targetPos2); i <= max(targetPos1, targetPos2); i++) {
      myVertex[i].draw();
    }

    //チールチップ生成のためのマウスと日ごとの円との衝突判定
    for (int i = min(targetPos1, targetPos2); i <= max(targetPos1, targetPos2); i++) {
      myVertex[i].collision();
    }
  }
}

void mousePressed() {

  //各パーツとマウスの衝突判定
  if (mouseX>firstPosX && mouseX < firstPosX+partsWidth) {
    if (mouseY>barPosY && mouseY < barPosY+barHeight) {
      isTarget1 = true;
    }
  }
  if (mouseX>secondPosX && mouseX < secondPosX+partsWidth) {
    if (mouseY>barPosY && mouseY < barPosY+barHeight) {
      isTarget2 = true;
    }
  }
}

void mouseDragged() {

  //パーツが重なりあった場合は１つ目のほうを使用する
  if (isOverlap) {
    isTarget1 = true;
    isTarget2 = false;
    isOverlap = false;
  }

  //ドラッグ時の各パーツを月ごとの大きさで移動するように指定
  if (isTarget1) {

    for (int i = barPosX; i < barWidth+barPosX; i+=partsWidth) {    
      if (mouseX > i && mouseX < i+partsWidth) {
        firstPosX = i;
        targetPos1 = (firstPosX-barPosX)/partsWidth;
      }
    }
  }
  if (isTarget2) {

    for (int i = barPosX; i < barWidth+barPosX; i+=partsWidth) {
      if (mouseX > i && mouseX < i+partsWidth) {
        secondPosX = i;
        targetPos2 = (secondPosX-barPosX)/partsWidth;
      }
    }
  }
}

void mouseReleased() {
  //フラグの解除
  isTarget1 = false;
  isTarget2 = false;
}

//月ごとのデータを描画するためのクラス
class MyVertex {
  Month month = new Month();//月ごとのデータ
  ArrayList <PVector>pos = new ArrayList <PVector>();//頂点の位置
  int max;//日数
  float mag = 4.8;
  int randomNum = (int)random(1, 6);//重なり合う線を描画する回数
  ArrayList <Float>randomPos = new ArrayList <Float>();

  //データを描画する基点
  float basePointX;
  float basePointY;

  MyVertex(Month _month) {
    month = _month;
    max = month.date.size();

    //頂点の位置を格納していく
    float angle = 0;
    for (int i = 0; i < month.date.size(); i++) {
      float temp = month.temp.get(i);

      float x = cos(radians(angle))*temp*1.3;
      float y = sin(radians(angle))*temp*1.3;
      pos.add(new PVector(x, y));
      float oneAngle = 360/ month.date.size();//日数で角度を決める
      angle += oneAngle;//角度を増やしていく
    }

    //重なりあう線の位置を生成
    for (int i = 0; i< randomNum; i++) {
      randomPos.add(random(0.98, 1.02));
    }

    basePointX = width/2;
    basePointY = height*0.44;
  }

  void draw() {
    noStroke();

    //日ごとに円を描画
    for (int i = 0; i < max; i++) {
      fill(createColor(month.temp.get(i), lowTemp, highTemp), 38, 99, 240);
      ellipse(pos.get(i).x*mag+basePointX, pos.get(i).y*mag+basePointY, 5, 5);
    }

    curveTightness(0.3);//曲線の曲率を指定
    beginShape();

    noFill();
    stroke(239, 38, 99, 100);

    //制御点のための最後の頂点から描画する
    curveVertex((pos.get(max-1).x*mag)+basePointX, (pos.get(max-1).y*mag)+basePointY);

    //順番に描画していく
    for (int i = 0; i < pos.size(); i++) {
      curveVertex((pos.get(i).x*mag)+basePointX, (pos.get(i).y*mag)+basePointY);
    }

    //閉曲線のため、初めの3点と同じ位置を繰り返す
    curveVertex((pos.get(0).x*mag)+basePointX, (pos.get(0).y*mag)+basePointY);
    curveVertex((pos.get(1).x*mag)+basePointX, (pos.get(1).y*mag)+basePointY);
    endShape();

    curveTightness(1.0);//曲線の曲率を指定

    //上記と同じく頂点を描画していき、ランダムで少しずらした線を描画する    
    for (int k = 0; k < randomNum; k++) {
      noFill();
      stroke(239, 38, 99, 70);
      beginShape();
      curveVertex(pos.get(max-1).x*mag*randomPos.get(k)+basePointX, pos.get(max-1).y*mag*randomPos.get(k)+basePointY);
      for (int i = 0; i < pos.size(); i++) {

        curveVertex(pos.get(i).x*mag*randomPos.get(k)+basePointX, pos.get(i).y*mag*randomPos.get(k)+basePointY);
      }
      curveVertex(pos.get(0).x*mag*randomPos.get(k)+basePointX, pos.get(0).y*mag*randomPos.get(k)+basePointY);
      curveVertex(pos.get(1).x*mag*randomPos.get(k)+basePointX, pos.get(1).y*mag*randomPos.get(k)+basePointY);
      endShape();
    }
  }

  //チールチップ描画のためのマウスとの衝突判定
  void collision() {

    //円の位置分ループさせる
    for (int i = 0; i < pos.size(); i++) {

      //円の位置とマウスの距離を取得
      float dist = dist(mouseX, mouseY, (pos.get(i).x*mag)+basePointX, (pos.get(i).y*mag)+basePointY);

      //5より下であればチールチップを出現させる
      if (dist < 5) {

        noStroke();
        fill(0, 0, 100, 255);
        int rectWidth = 50;

        //画面の半分より上
        if (mouseY > height/2) {

          //画面の左端
          if (mouseX < height/2*0.5) {//グラフ左端
            rect(mouseX-65, mouseY+15, rectWidth, -30);
            fill(0);            
            text(month.date.get(i), mouseX-40, mouseY-2);          
            text(month.temp.get(i).toString()+ "℃", mouseX-40, mouseY+10);
          } else if (mouseX > height/2*1.7) {//グラフ右端
            rect(mouseX+65, mouseY+15, -rectWidth, -30);
            fill(0);
            text(month.date.get(i), mouseX+40, mouseY-2);
            text(month.temp.get(i).toString()+ "℃", mouseX+40, mouseY+10);
          } else {
            rect(mouseX-rectWidth/2, mouseY+20, rectWidth, 30);
            fill(0);            
            text(month.date.get(i), mouseX, mouseY+34);
            text(month.temp.get(i).toString()+ "℃", mouseX, mouseY+45);
          }
        }

        //画面の半分より下
        if (mouseY < height/2) {

          if (mouseX < height/2*0.5) {//グラフ左端
            rect(mouseX-65, mouseY+15, rectWidth, -30);
            fill(0);
            text(month.date.get(i), mouseX-40, mouseY-2);
            text(month.temp.get(i).toString()+ "℃", mouseX-40, mouseY+10);
          } else if (mouseX > height/2*1.7) {//グラフ右端
            rect(mouseX+65, mouseY+15, -rectWidth, -30);
            fill(0);
            text(month.date.get(i), mouseX+40, mouseY-2);
            text(month.temp.get(i).toString()+ "℃", mouseX+40, mouseY+10);
          } else {
            rect(mouseX-rectWidth/2, mouseY-10, rectWidth, -30);
            fill(0);
            text(month.date.get(i), mouseX, mouseY-26);
            text(month.temp.get(i).toString()+ "℃", mouseX, mouseY-14);
          }
        }
      }
    }
  }
}

//月ごとのデータを格納する
class Month {
  ArrayList<String> date = new ArrayList<String>();//日付
  ArrayList<Float> temp = new ArrayList<Float>();//気温

  void setNum(String _date, float _temp) {
    date.add(_date);
    temp.add(_temp);
  }
}

void loadData() {
  //ファイルをロードして配列を作る
  JSONArray weatherData = loadJSONArray("weather_day.json");

  //要素数を取得して配列を生成
  date = new String[weatherData.size()];
  temp = new float[weatherData.size()];

  for (int i = 0; i < weatherData.size(); i++) {

    //一つ一つのJSON配列の中を検索
    JSONObject weather = weatherData.getJSONObject(i);

    //date, temp配列に一つずつ入れる
    date[i] = weather.getString("date");
    temp[i] = weather.getFloat("temp");
  }
}

//気温から色を決定
float createColor(float _temp, float _lowTemp, float _highTemp) {

  //最低気温から最高気温の範囲を、色相の200°から0°に変換
  //気温を色に変換
  float barColor = map(_temp, _lowTemp, _highTemp, 200, 0);

  return barColor;
}